import os
import sys
from miquido_license_validator import license_validator

ignore_unknown = os.getenv('IGNORE_UNKNOWN', False)


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


if __name__ == '__main__':
    if len(sys.argv) != 2:
        eprint('Missing file argument')
        exit(1)

    result = license_validator.are_valid(sys.argv[1])

    if len(result['notok']) > 0:
        eprint('Not ok licenses found')
        eprint(result['notok'])
        exit(1)

    if not ignore_unknown and len(result['unknown']) > 0:
        eprint('Unknown licenses found')
        eprint(result['unknown'])
        exit(1)



import json
import requests
from functools import reduce

from miquido_license_validator import common


def __reduce(ok_licences, notok_licenses, dependencies):
    ignored_licenses = __get_ignored()

    def inner_reduce(res, x):
        if x.lower() in ignored_licenses:
            return res
        elem = {'name': x, 'dependencies': dependencies[x]}
        if x.lower() in ok_licences:
            res['ok'].append(elem)
        elif x.lower() in notok_licenses:
            res['notok'].append(elem)
        else:
            res['unknown'].append(elem)
        return res

    return inner_reduce


def __get_ignored():
    try:
        with open('.licenseignore', 'r') as f:
            ignored_licenses = f.read().split('\n')

        return set(map(lambda x: x.lower(), ignored_licenses))

    except FileNotFoundError:
        return set()


def are_valid(license_file):
    with open(license_file, 'r') as f:
        licenses = json.loads(f.read())

    license_reference = requests.get('https://licenses.miquido.dev').json()
    ok_licenses = set(map(lambda x: x.lower(), license_reference['ok']))
    notok_licenses = set(map(lambda x: x.lower(), license_reference['notok']))
    result = {
        'ok': [],
        'notok': [],
        'unknown': []
    }

    dependencies = common.flatten_license_dependencies(licenses['dependencies'])

    licenses_names = list(map(lambda x: x['id'], licenses['licenses']))

    return reduce(__reduce(ok_licenses, notok_licenses, dependencies), licenses_names, result)



import json
import os
import unittest
from unittest import mock
from miquido_license_validator import license_validator


class MockResponse:
    def __init__(self, status_code, json_body: dict):
        self.json_body = json_body
        self.status_code = status_code

    def json(self):
        return self.json_body


def mocked_request(*args, **kwargs):
    with open('test/fixtures/licenses_reference.json', 'r') as file:
        res = file.read()
    return MockResponse(200, json.loads(res))


class LicenseVerifierTests(unittest.TestCase):

    @mock.patch('requests.get', side_effect=mocked_request)
    def test_license_verifier(self, mock):
        res = license_validator.are_valid('test/fixtures/gl-license-scanning-report.json')
        self.assertEqual(1, len(res['ok']))
        self.assertEqual(2, len(res['notok']))
        self.assertEqual(5, len(res['unknown']))
        self.assertEqual("MIT", res['ok'][0]['name'])
        self.assertEqual("bsd", res['notok'][0]['name'])
        self.assertEqual("GNU", res['notok'][1]['name'])
        self.assertEqual(2, len(res['notok'][1]['dependencies']))
        self.assertListEqual(['certifi', 'cffi'], res['notok'][1]['dependencies'])

    @mock.patch('requests.get', side_effect=mocked_request)
    def test_ignores(self, mock):
        with open('test/fixtures/ignorelist.txt', 'r') as f:
            ignores = f.read()
        with open('.licenseignore', 'w') as f:
            f.write(ignores)

        res = license_validator.are_valid('test/fixtures/gl-license-scanning-report.json')

        self.assertEqual(1, len(res['ok']))
        self.assertEqual(1, len(res['notok']))
        self.assertEqual(3, len(res['unknown']))
        self.assertEqual("MIT", res['ok'][0]['name'])
        self.assertEqual("GNU", res['notok'][0]['name'])

    @classmethod
    def setUpClass(cls) -> None:
        with open('.licenseignore', 'r') as f:
            cls.actual_ignore = f.read()
        try:
            os.remove(".licenseignore")
        except FileNotFoundError:
            pass

    @classmethod
    def tearDownClass(cls) -> None:
        with open('.licenseignore', 'w') as f:
            f.write(cls.actual_ignore)

    def setUp(self) -> None:
        pass

    def tearDown(self) -> None:
        try:
            os.remove(".licenseignore")
        except FileNotFoundError:
            pass



import unittest

from miquido_license_validator import common

dependencies = [
    {
        "name": "PyYAML",
        "version": "5.4.1",
        "package_manager": "pip",
        "path": "requirements.txt",
        "licenses": [
            "LIC_1",
            "LIC_2"
        ]
    },
    {
        "name": "SecretStorage",
        "version": "3.3.1",
        "package_manager": "pip",
        "path": "requirements.txt",
        "licenses": [
            "LIC_1"
        ]
    },
    {
        "name": "cachetools",
        "version": "4.2.4",
        "package_manager": "pip",
        "path": "requirements.txt",
        "licenses": [
            "LIC_3"
        ]
    }
]


class LicenseDependencyFlattenTests(unittest.TestCase):
    def test_flatten(self):
        flatten = common.flatten_license_dependencies(dependencies)
        self.assertListEqual(['PyYAML', 'SecretStorage'], flatten['LIC_1'])
        self.assertListEqual(['PyYAML'], flatten['LIC_2'])
        self.assertListEqual(['cachetools'], flatten['LIC_3'])



from functools import reduce


def __reduce(res, x):
    for lic in x['licenses']:
        if lic not in res:
            res[lic] = []
        res[lic].append(x['name'])
    return res


def flatten_license_dependencies(dependencies):
    return reduce(__reduce, dependencies, {})